@@ -15,12 +15,15 @@
 // specific language governing permissions and limitations
 // under the License.
 
+use crate::jvm_bridge::jni_new_global_ref;
 use crate::{
     errors::CometError,
     jvm_bridge::{jni_call, jni_new_string},
 };
 use datafusion::physical_plan::ExecutionPlan;
+use jni::objects::{GlobalRef, JString};
 use jni::{objects::JObject, JNIEnv};
+use std::collections::HashMap;
 use std::sync::Arc;
 
 /// Updates the metrics of a CometMetricNode. This function is called recursively to
@@ -30,6 +33,7 @@ pub fn update_comet_metric(
     env: &mut JNIEnv,
     metric_node: &JObject,
     execution_plan: &Arc<dyn ExecutionPlan>,
+    metrics_jstrings: &mut HashMap<String, Arc<GlobalRef>>,
 ) -> Result<(), CometError> {
     update_metrics(
         env,
@@ -41,6 +45,7 @@ pub fn update_comet_metric(
             .map(|m| m.value())
             .map(|m| (m.name(), m.as_usize() as i64))
             .collect::<Vec<_>>(),
+        metrics_jstrings,
     )?;
 
     unsafe {
@@ -51,7 +56,7 @@ pub fn update_comet_metric(
             if child_metric_node.is_null() {
                 continue;
             }
-            update_comet_metric(env, &child_metric_node, child_plan)?;
+            update_comet_metric(env, &child_metric_node, child_plan, metrics_jstrings)?;
         }
     }
     Ok(())
@@ -62,11 +67,28 @@ fn update_metrics(
     env: &mut JNIEnv,
     metric_node: &JObject,
     metric_values: &[(&str, i64)],
+    metrics_jstrings: &mut HashMap<String, Arc<GlobalRef>>,
 ) -> Result<(), CometError> {
     unsafe {
         for &(name, value) in metric_values {
-            let jname = jni_new_string!(env, &name)?;
-            jni_call!(env, comet_metric_node(metric_node).set(&jname, value) -> ())?;
+            // Perform a lookup in the jstrings cache.
+            if let Some(map_global_ref) = metrics_jstrings.get(name) {
+                // Cache hit. Extract the jstring from the global ref.
+                let jobject = map_global_ref.as_obj();
+                let jstring = JString::from_raw(**jobject);
+                // Update the metrics using the jstring as a key.
+                jni_call!(env, comet_metric_node(metric_node).set(&jstring, value) -> ())?;
+            } else {
+                // Cache miss. Allocate a new string, promote to global ref, and insert into cache.
+                let local_jstring = jni_new_string!(env, &name)?;
+                let global_ref = jni_new_global_ref!(env, local_jstring)?;
+                let arc_global_ref = Arc::new(global_ref);
+                metrics_jstrings.insert(name.to_string(), Arc::clone(&arc_global_ref));
+                let jobject = arc_global_ref.as_obj();
+                let jstring = JString::from_raw(**jobject);
+                // Update the metrics using the jstring as a key.
+                jni_call!(env, comet_metric_node(metric_node).set(&jstring, value) -> ())?;
+            }
         }
     }
     Ok(())
