@@ -27,7 +27,7 @@ use std::{boxed::Box, ptr::NonNull, sync::Arc};
 
 use crate::errors::{try_unwrap_or_throw, CometError};
 
-use arrow::ffi::{FFI_ArrowArray, FFI_ArrowSchema};
+use arrow::ffi::FFI_ArrowArray;
 
 /// JNI exposed methods
 use jni::JNIEnv;
@@ -52,7 +52,6 @@ const STR_CLASS_NAME: &str = "java/lang/String";
 /// Parquet read context maintained across multiple JNI calls.
 struct Context {
     pub column_reader: ColumnReader,
-    pub arrays: Option<(Arc<FFI_ArrowArray>, Arc<FFI_ArrowSchema>)>,
     last_data_page: Option<GlobalRef>,
 }
 
@@ -110,7 +109,6 @@ pub extern "system" fn Java_org_apache_comet_parquet_Native_initColumnReader(
                 use_decimal_128 != 0,
                 use_legacy_date_timestamp != 0,
             ),
-            arrays: None,
             last_data_page: None,
         };
         let res = Box::new(ctx);
@@ -539,24 +537,15 @@ pub extern "system" fn Java_org_apache_comet_parquet_Native_currentBatch(
     e: JNIEnv,
     _jclass: JClass,
     handle: jlong,
-) -> jlongArray {
-    try_unwrap_or_throw(&e, |env| {
+    array_addr: jlong,
+    schema_addr: jlong,
+) {
+    try_unwrap_or_throw(&e, |_env| {
         let ctx = get_context(handle)?;
         let reader = &mut ctx.column_reader;
         let data = reader.current_batch();
-        let (array, schema) = data.to_spark()?;
-
-        unsafe {
-            let arrow_array = Arc::from_raw(array as *const FFI_ArrowArray);
-            let arrow_schema = Arc::from_raw(schema as *const FFI_ArrowSchema);
-            ctx.arrays = Some((arrow_array, arrow_schema));
-
-            let res = env.new_long_array(2)?;
-            let buf: [i64; 2] = [array, schema];
-            env.set_long_array_region(&res, 0, &buf)
-                .expect("set long array region failed");
-            Ok(res.into_raw())
-        }
+        data.move_to_spark(array_addr, schema_addr)
+            .map_err(|e| e.into())
     })
 }
 
