@@ -19,7 +19,7 @@
 
 package org.apache.comet.expressions
 
-import org.apache.spark.sql.types.{DataType, DataTypes, DecimalType}
+import org.apache.spark.sql.types.{DataType, DataTypes, DecimalType, StructType}
 
 sealed trait SupportLevel
 
@@ -76,7 +76,7 @@ object CometCast {
       case (DataTypes.StringType, _) =>
         canCastFromString(toType, timeZoneId, evalMode)
       case (_, DataTypes.StringType) =>
-        canCastToString(fromType)
+        canCastToString(fromType, timeZoneId, evalMode)
       case (DataTypes.TimestampType, _) =>
         canCastFromTimestamp(toType)
       case (_: DecimalType, _) =>
@@ -95,6 +95,16 @@ object CometCast {
         canCastFromFloat(toType)
       case (DataTypes.DoubleType, _) =>
         canCastFromDouble(toType)
+      case (from_struct: StructType, to_struct: StructType) =>
+        from_struct.fields.zip(to_struct.fields).foreach { case (a, b) =>
+          isSupported(a.dataType, b.dataType, timeZoneId, evalMode) match {
+            case Compatible(_) =>
+            // all good
+            case other =>
+              return other
+          }
+        }
+        Compatible()
       case _ => Unsupported
     }
   }
@@ -137,7 +147,10 @@ object CometCast {
     }
   }
 
-  private def canCastToString(fromType: DataType): SupportLevel = {
+  private def canCastToString(
+      fromType: DataType,
+      timeZoneId: Option[String],
+      evalMode: CometEvalMode.Value): SupportLevel = {
     fromType match {
       case DataTypes.BooleanType => Compatible()
       case DataTypes.ByteType | DataTypes.ShortType | DataTypes.IntegerType |
@@ -151,9 +164,26 @@ object CometCast {
             "There can be differences in precision. " +
               "For example, the input \"1.4E-45\" will produce 1.0E-45 " +
               "instead of 1.4E-45"))
+      case _: DecimalType =>
+        // https://github.com/apache/datafusion-comet/issues/1068
+        Compatible(
+          Some(
+            "There can be formatting differences in some case due to Spark using " +
+              "scientific notation where Comet does not"))
       case DataTypes.BinaryType =>
         // https://github.com/apache/datafusion-comet/issues/377
         Incompatible(Some("Only works for binary data representing valid UTF-8 strings"))
+      case StructType(fields) =>
+        for (field <- fields) {
+          isSupported(field.dataType, DataTypes.StringType, timeZoneId, evalMode) match {
+            case s: Incompatible =>
+              return s
+            case Unsupported =>
+              return Unsupported
+            case _ =>
+          }
+        }
+        Compatible()
       case _ => Unsupported
     }
   }
