@@ -19,8 +19,6 @@
 
 package org.apache.spark.sql.comet
 
-import scala.concurrent.duration.NANOSECONDS
-
 import org.apache.spark.rdd._
 import org.apache.spark.sql.catalyst._
 import org.apache.spark.sql.catalyst.expressions.{Attribute, DynamicPruningExpression, Expression, Literal}
@@ -60,7 +58,7 @@ case class CometBatchScanExec(wrapped: BatchScanExec, runtimeFilters: Seq[Expres
           // The `FileScanRDD` returns an iterator which scans the file during the `hasNext` call.
           val startNs = System.nanoTime()
           val res = batches.hasNext
-          scanTime += NANOSECONDS.toMillis(System.nanoTime() - startNs)
+          scanTime += System.nanoTime() - startNs
           res
         }
 
@@ -139,7 +137,7 @@ case class CometBatchScanExec(wrapped: BatchScanExec, runtimeFilters: Seq[Expres
 
   override lazy val metrics: Map[String, SQLMetric] = Map(
     "numOutputRows" -> SQLMetrics.createMetric(sparkContext, "number of output rows"),
-    "scanTime" -> SQLMetrics.createTimingMetric(
+    "scanTime" -> SQLMetrics.createNanoTimingMetric(
       sparkContext,
       "scan time")) ++ wrapped.customMetrics ++ {
     wrapped.scan match {
